"use strict";
var Fiber = require("fibers");
var Future = require("fibers/future");
var iconv = require("iconv-lite");
var os = require("os");
var osenv = require("osenv");
var path = require("path");
var util = require("util");
var MobileHelper = require("../mobile-helper");
var options = require("../../options");
var helpers = require("../../helpers");
var net = require('net');
var VirtualMachine = (function () {
    function VirtualMachine(name, identifier) {
        this.name = name;
        this.identifier = identifier;
    }
    return VirtualMachine;
})();
var AndroidEmulatorServices = (function () {
    function AndroidEmulatorServices($logger, $emulatorSettingsService, $errors, $childProcess, $fs, $staticConfig) {
        this.$logger = $logger;
        this.$emulatorSettingsService = $emulatorSettingsService;
        this.$errors = $errors;
        this.$childProcess = $childProcess;
        this.$fs = $fs;
        this.$staticConfig = $staticConfig;
        iconv.extendNodeEncodings();
        this.adbFilePath = helpers.getPathToAdb($injector).wait();
    }
    AndroidEmulatorServices.prototype.checkDependencies = function () {
        var _this = this;
        return (function () {
            _this.checkAndroidSDKConfiguration().wait();
            if (options.geny) {
                _this.checkGenymotionConfiguration().wait();
            }
        }).future()();
    };
    AndroidEmulatorServices.prototype.checkAndroidSDKConfiguration = function () {
        var _this = this;
        return (function () {
            try {
                var proc = _this.$childProcess.spawnFromEvent('emulator', ['-help'], "exit", undefined, { throwError: false }).wait();
                if (proc.stderr) {
                    _this.$errors.fail({ formatStr: AndroidEmulatorServices.MISSING_SDK_MESSAGE, suppressCommandHelp: true });
                }
            }
            catch (e) {
                var message = (e.code === "ENOENT") ? AndroidEmulatorServices.MISSING_SDK_MESSAGE : e.message;
                _this.$errors.failWithoutHelp(message);
            }
        }).future()();
    };
    AndroidEmulatorServices.prototype.checkGenymotionConfiguration = function () {
        var _this = this;
        return (function () {
            try {
                var proc = _this.$childProcess.spawnFromEvent("player", [], "exit", undefined, { throwError: false }).wait();
            }
            catch (e) {
                var message = (e.code === "ENOENT") ? AndroidEmulatorServices.MISSING_GENYMOTION_MESSAGE : e.message;
                _this.$errors.failWithoutHelp(message);
            }
            if (proc.stderr && !proc.stderr.startsWith("Usage:")) {
                _this.$errors.failWithoutHelp(AndroidEmulatorServices.MISSING_GENYMOTION_MESSAGE);
            }
        }).future()();
    };
    AndroidEmulatorServices.prototype.checkAvailability = function () {
        var _this = this;
        return (function () {
            var platform = MobileHelper.DevicePlatforms[1 /* Android */];
            if (!_this.$emulatorSettingsService.canStart(platform).wait()) {
                _this.$errors.fail("The current project does not target Android and cannot be run in the Android emulator.");
            }
        }).future()();
    };
    AndroidEmulatorServices.prototype.startEmulator = function (app, emulatorOptions) {
        var _this = this;
        return (function () {
            if (options.avd && options.geny) {
                _this.$errors.fail("You cannot specify both --avd and --geny options. Please use only one of them.");
            }
            var image = options.avd || options.geny || _this.getBestFit().wait();
            if (image) {
                _this.startEmulatorCore(app, emulatorOptions.appId, image).wait();
            }
            else {
                _this.$errors.fail("Could not find an emulator image to run your project.");
            }
        }).future()();
    };
    AndroidEmulatorServices.prototype.startEmulatorCore = function (app, appId, image) {
        var _this = this;
        return (function () {
            var emulatorId = _this.startEmulatorInstance(image).wait();
            _this.waitForEmulatorBootToComplete(emulatorId).wait();
            _this.unlockScreen(emulatorId).wait();
            _this.$logger.info("installing %s through adb", app);
            var childProcess = _this.$childProcess.spawn(_this.adbFilePath, ["-s", emulatorId, 'install', '-r', app]);
            _this.$fs.futureFromEvent(childProcess, "close").wait();
            _this.unlockScreen(emulatorId).wait();
            _this.$logger.info("running %s through adb", app);
            childProcess = _this.$childProcess.spawn(_this.adbFilePath, ["-s", emulatorId, 'shell', 'am', 'start', '-S', appId + "/" + _this.$staticConfig.START_PACKAGE_ACTIVITY_NAME], { stdio: "ignore", detached: true });
            _this.$fs.futureFromEvent(childProcess, "close").wait();
        }).future()();
    };
    AndroidEmulatorServices.prototype.unlockScreen = function (emulatorId) {
        var childProcess = this.$childProcess.spawn(this.adbFilePath, ["-s", emulatorId, "shell", "input", "keyevent", "82"]);
        return this.$fs.futureFromEvent(childProcess, "close");
    };
    AndroidEmulatorServices.prototype.sleep = function (ms) {
        var fiber = Fiber.current;
        setTimeout(function () { return fiber.run(); }, ms);
        Fiber.yield();
    };
    AndroidEmulatorServices.prototype.getMilliSecondsTimeout = function () {
        var timeout = AndroidEmulatorServices.TIMEOUT_SECONDS;
        if (options && options.timeout) {
            var parsedValue = parseInt(options.timeout);
            if (!isNaN(parsedValue) && parsedValue >= 0) {
                timeout = parsedValue;
            }
            else {
                this.$logger.info("Specify timeout in a number of seconds to wait. Set it to 0 to wait indefinitely. Default value: " + timeout + " seconds will be used.");
            }
        }
        return timeout * 1000;
    };
    AndroidEmulatorServices.prototype.getRunningEmulatorId = function (image) {
        var _this = this;
        return (function () {
            var runningEmulators = _this.getRunningEmulators().wait();
            if (runningEmulators.length === 0) {
                return "";
            }
            var getNameFunction = options.geny ? _this.getNameFromGenymotionEmulatorId : _this.getNameFromSDKEmulatorId;
            var emulatorId = _(runningEmulators).find(function (emulator) { return getNameFunction.apply(_this, [emulator]).wait() === image; });
            return emulatorId;
        }).future()();
    };
    AndroidEmulatorServices.prototype.getNameFromGenymotionEmulatorId = function (emulatorId) {
        var _this = this;
        return (function () {
            var modelOutputLines = _this.$childProcess.execFile(_this.adbFilePath, ["-s", emulatorId, "shell", "getprop", "ro.product.model"]).wait();
            _this.$logger.trace(modelOutputLines);
            var model = _.first(modelOutputLines.split(os.EOL)).trim();
            return model;
        }).future()();
    };
    AndroidEmulatorServices.prototype.getNameFromSDKEmulatorId = function (emulatorId) {
        var match = emulatorId.match(/^emulator-(\d+)/);
        var portNumber;
        if (match && match[1]) {
            portNumber = match[1];
        }
        else {
            return Future.fromResult("");
        }
        var future = new Future();
        var output = "";
        var client = net.connect({ port: portNumber }, function () {
            client.write(util.format("avd name%s", os.EOL));
        });
        client.on('data', function (data) {
            output += data.toString();
            client.end();
        });
        client.on('end', function () {
            var name;
            var foundOK = false;
            var lines = output.split(os.EOL);
            _(lines).each(function (line) {
                if (foundOK) {
                    name = line.trim();
                    return false;
                }
                else if (line.match(/^OK/)) {
                    foundOK = true;
                }
            }).value();
            future.return(name);
        });
        return future;
    };
    AndroidEmulatorServices.prototype.startEmulatorInstance = function (image) {
        var _this = this;
        return (function () {
            var emulatorId = _this.getRunningEmulatorId(image).wait();
            _this.endTimeEpoch = helpers.getCurrentEpochTime() + _this.getMilliSecondsTimeout();
            if (emulatorId) {
                return emulatorId;
            }
            _this.$logger.info("Starting Android emulator with image %s", image);
            if (options.geny) {
                _this.$childProcess.spawn("player", ["--vm-name", image], { stdio: "ignore", detached: true }).unref();
            }
            else {
                _this.$childProcess.spawn('emulator', ['-avd', image], { stdio: "ignore", detached: true }).unref();
            }
            var isInfiniteWait = _this.getMilliSecondsTimeout() === 0;
            var hasTimeLeft = helpers.getCurrentEpochTime() < _this.endTimeEpoch;
            while (hasTimeLeft || isInfiniteWait) {
                emulatorId = _this.getRunningEmulatorId(image).wait();
                if (emulatorId) {
                    return emulatorId;
                }
                _this.sleep(10000);
                hasTimeLeft = helpers.getCurrentEpochTime() < _this.endTimeEpoch;
            }
            if (!hasTimeLeft && !isInfiniteWait) {
                _this.$errors.fail(AndroidEmulatorServices.UNABLE_TO_START_EMULATOR_MESSAGE);
            }
            return emulatorId;
        }).future()();
    };
    AndroidEmulatorServices.prototype.getRunningGenymotionEmulators = function (adbDevicesOutput) {
        var _this = this;
        return (function () {
            var futures = _(adbDevicesOutput).filter(function (r) { return !r.match(AndroidEmulatorServices.RUNNING_ANDROID_EMULATOR_REGEX); }).map(function (row) {
                var match = row.match(/^(.+?)\s+device$/);
                if (match && match[1]) {
                    var emulatorId = match[1];
                    return _this.checkForGenymotionProductManufacturer(emulatorId);
                }
                return Future.fromResult(undefined);
            }).value();
            Future.wait(futures);
            return _(futures).filter(function (future) { return !!future.get(); }).map(function (f) { return f.get().toString(); }).value();
        }).future()();
    };
    AndroidEmulatorServices.prototype.checkForGenymotionProductManufacturer = function (emulatorId) {
        var _this = this;
        return (function () {
            var manufacturer = _this.$childProcess.execFile(_this.adbFilePath, ["-s", emulatorId, "shell", "getprop", "ro.product.manufacturer"]).wait();
            if (manufacturer.match(/^Genymotion/i)) {
                return emulatorId;
            }
            return undefined;
        }).future()();
    };
    AndroidEmulatorServices.prototype.getRunningEmulators = function () {
        var _this = this;
        return (function () {
            var emulatorDevices = [];
            var outputRaw = _this.$childProcess.execFile(_this.adbFilePath, ['devices']).wait().split(os.EOL);
            if (options.geny) {
                emulatorDevices = _this.getRunningGenymotionEmulators(outputRaw).wait();
            }
            else {
                _.each(outputRaw, function (device) {
                    var rx = device.match(AndroidEmulatorServices.RUNNING_ANDROID_EMULATOR_REGEX);
                    if (rx && rx[1]) {
                        emulatorDevices.push(rx[1]);
                    }
                });
            }
            return emulatorDevices;
        }).future()();
    };
    AndroidEmulatorServices.prototype.getBestFit = function () {
        var _this = this;
        return (function () {
            var minVersion = _this.$emulatorSettingsService.minVersion;
            var best = _(_this.getAvds().wait()).map(function (avd) { return _this.getInfoFromAvd(avd).wait(); }).max(function (avd) { return avd.targetNum; });
            return (best.targetNum >= minVersion) ? best.name : null;
        }).future()();
    };
    AndroidEmulatorServices.prototype.getInfoFromAvd = function (avdName) {
        var _this = this;
        return (function () {
            var iniFile = path.join(_this.avdDir, avdName + ".ini");
            var avdInfo = _this.parseAvdFile(avdName, iniFile).wait();
            if (avdInfo.path && _this.$fs.exists(avdInfo.path).wait()) {
                iniFile = path.join(avdInfo.path, "config.ini");
                avdInfo = _this.parseAvdFile(avdName, iniFile, avdInfo).wait();
            }
            return avdInfo;
        }).future()();
    };
    AndroidEmulatorServices.prototype.parseAvdFile = function (avdName, avdFileName, avdInfo) {
        var _this = this;
        if (avdInfo === void 0) { avdInfo = null; }
        return (function () {
            var encoding = _this.getAvdEncoding(avdFileName).wait();
            var contents = _this.$fs.readText(avdFileName, encoding).wait().split("\n");
            avdInfo = _.reduce(contents, function (result, line) {
                var parsedLine = line.split("=");
                var key = parsedLine[0];
                switch (key) {
                    case "target":
                        result.target = parsedLine[1];
                        result.targetNum = _this.readTargetNum(result.target);
                        break;
                    case "path":
                        result.path = parsedLine[1];
                        break;
                    case "hw.device.name":
                        result.device = parsedLine[1];
                        break;
                    case "abi.type":
                        result.abi = parsedLine[1];
                        break;
                    case "skin.name":
                        result.skin = parsedLine[1];
                        break;
                    case "sdcard.size":
                        result.sdcard = parsedLine[1];
                        break;
                }
                return result;
            }, avdInfo || Object.create(null));
            avdInfo.name = avdName;
            return avdInfo;
        }).future()();
    };
    AndroidEmulatorServices.prototype.readTargetNum = function (target) {
        var platform = target.replace('android-', '');
        var platformNumber = +platform;
        if (isNaN(platformNumber)) {
            if (platform === "L") {
                platformNumber = 20;
            }
        }
        return platformNumber;
    };
    AndroidEmulatorServices.prototype.getAvdEncoding = function (avdName) {
        var _this = this;
        return (function () {
            var encoding = "utf8";
            var contents = _this.$fs.readText(avdName, "ascii").wait();
            if (contents.length > 0) {
                contents = contents.split("\n", 1)[0];
                if (contents.length > 0) {
                    var matches = contents.match(AndroidEmulatorServices.ENCODING_MASK);
                    if (matches) {
                        encoding = matches[1];
                    }
                }
            }
            return encoding;
        }).future()();
    };
    Object.defineProperty(AndroidEmulatorServices.prototype, "androidHomeDir", {
        get: function () {
            return path.join(osenv.home(), AndroidEmulatorServices.ANDROID_DIR_NAME);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AndroidEmulatorServices.prototype, "avdDir", {
        get: function () {
            return path.join(this.androidHomeDir, AndroidEmulatorServices.AVD_DIR_NAME);
        },
        enumerable: true,
        configurable: true
    });
    AndroidEmulatorServices.prototype.getAvds = function () {
        var _this = this;
        return (function () {
            var result = [];
            if (_this.$fs.exists(_this.avdDir).wait()) {
                var entries = _this.$fs.readDirectory(_this.avdDir).wait();
                result = _.select(entries, function (e) { return e.match(AndroidEmulatorServices.INI_FILES_MASK) !== null; }).map(function (e) { return e.match(AndroidEmulatorServices.INI_FILES_MASK)[1]; });
            }
            return result;
        }).future()();
    };
    AndroidEmulatorServices.prototype.waitForEmulatorBootToComplete = function (emulatorId) {
        var _this = this;
        return (function () {
            _this.$logger.printInfoMessageOnSameLine("Waiting for emulator device initialization...");
            var isInfiniteWait = _this.getMilliSecondsTimeout() === 0;
            while (helpers.getCurrentEpochTime() < _this.endTimeEpoch || isInfiniteWait) {
                var isEmulatorBootCompleted = _this.isEmulatorBootCompleted(emulatorId).wait();
                if (isEmulatorBootCompleted) {
                    _this.$logger.printInfoMessageOnSameLine(os.EOL);
                    return;
                }
                _this.$logger.printInfoMessageOnSameLine(".");
                _this.sleep(10000);
            }
            _this.$logger.printInfoMessageOnSameLine(os.EOL);
            _this.$errors.fail(AndroidEmulatorServices.UNABLE_TO_START_EMULATOR_MESSAGE);
        }).future()();
    };
    AndroidEmulatorServices.prototype.isEmulatorBootCompleted = function (emulatorId) {
        var _this = this;
        return (function () {
            var output = _this.$childProcess.execFile(_this.adbFilePath, ["-s", emulatorId, "shell", "getprop", "dev.bootcomplete"]).wait();
            var matches = output.match("1");
            return matches && matches.length > 0;
        }).future()();
    };
    AndroidEmulatorServices.ANDROID_DIR_NAME = ".android";
    AndroidEmulatorServices.AVD_DIR_NAME = "avd";
    AndroidEmulatorServices.INI_FILES_MASK = /^(.*)\.ini$/i;
    AndroidEmulatorServices.ENCODING_MASK = /^avd\.ini\.encoding=(.*)$/;
    AndroidEmulatorServices.TIMEOUT_SECONDS = 120;
    AndroidEmulatorServices.UNABLE_TO_START_EMULATOR_MESSAGE = "Cannot run your app in the native emulator. Increase the timeout of the operation with the --timeout option or try to restart your adb server with 'adb kill-server' command. Alternatively, run the Android Virtual Device manager and increase the allocated RAM for the virtual device.";
    AndroidEmulatorServices.RUNNING_ANDROID_EMULATOR_REGEX = /^(emulator-\d+)\s+device$/;
    AndroidEmulatorServices.MISSING_SDK_MESSAGE = "The Android SDK is not configured properly. " + "Verify that you have installed the Android SDK and that you have added its `platform-tools` and `tools` directories to your PATH environment variable.";
    AndroidEmulatorServices.MISSING_GENYMOTION_MESSAGE = "Genymotion is not configured properly. " + "Verify that you have installed Genymotion and that you have added its installation directory to your PATH environment variable.";
    return AndroidEmulatorServices;
})();
$injector.register("androidEmulatorServices", AndroidEmulatorServices);
