"use strict";
var Future = require("fibers/future");
var npm = require("npm");
var path = require("path");
var semver = require("semver");
var constants = require("./constants");
var options = require("./common/options");
var NodePackageManager = (function () {
    function NodePackageManager($logger, $errors, $httpClient, $fs, $lockfile) {
        this.$logger = $logger;
        this.$errors = $errors;
        this.$httpClient = $httpClient;
        this.$fs = $fs;
        this.$lockfile = $lockfile;
        this.versionsCache = {};
        this.load().wait();
    }
    NodePackageManager.prototype.getCacheRootPath = function () {
        return npm.cache;
    };
    NodePackageManager.prototype.addToCache = function (packageName, version) {
        return this.addToCacheCore(packageName, version);
    };
    NodePackageManager.prototype.load = function (config) {
        var future = new Future();
        npm.load(config, function (err) {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    };
    NodePackageManager.prototype.install = function (packageName, opts) {
        var _this = this;
        return (function () {
            _this.$lockfile.lock().wait();
            try {
                var packageToInstall = packageName;
                var pathToSave = (opts && opts.pathToSave) || npm.cache;
                var version = (opts && opts.version) || null;
                return _this.installCore(packageToInstall, pathToSave, version).wait();
            }
            catch (error) {
                _this.$logger.debug(error);
                _this.$errors.fail("%s. Error: %s", NodePackageManager.NPM_LOAD_FAILED, error);
            }
            finally {
                _this.$lockfile.unlock().wait();
            }
        }).future()();
    };
    NodePackageManager.prototype.getLatestVersion = function (packageName) {
        var _this = this;
        return (function () {
            var versions = _this.getAvailableVersions(packageName).wait();
            versions = _.sortBy(versions, function (ver) {
                return ver;
            });
            return versions.reverse()[0];
        }).future()();
    };
    NodePackageManager.prototype.installCore = function (packageName, pathToSave, version) {
        var _this = this;
        return (function () {
            if (options.frameworkPath) {
                if (_this.$fs.getFsStats(options.frameworkPath).wait().isFile()) {
                    _this.npmInstall(packageName, pathToSave, version).wait();
                    var pathToNodeModules = path.join(pathToSave, "node_modules");
                    var folders = _this.$fs.readDirectory(pathToNodeModules).wait();
                    return path.join(pathToNodeModules, folders[0]);
                }
                return options.frameworkPath;
            }
            else {
                var version = version || _this.getLatestVersion(packageName).wait();
                var packagePath = path.join(npm.cache, packageName, version, "package");
                if (!_this.isPackageCached(packagePath).wait()) {
                    _this.addToCacheCore(packageName, version).wait();
                }
                if (!_this.isPackageUnpacked(packagePath).wait()) {
                    _this.cacheUnpack(packageName, version).wait();
                }
                return packagePath;
            }
        }).future()();
    };
    NodePackageManager.prototype.npmInstall = function (packageName, pathToSave, version) {
        var _this = this;
        this.$logger.out("Installing ", packageName);
        var incrementedVersion = semver.inc(version, constants.ReleaseType.MINOR);
        if (!options.frameworkPath && packageName.indexOf("@") < 0) {
            packageName = packageName + "@<" + incrementedVersion;
        }
        var future = new Future();
        npm.commands["install"](pathToSave, packageName, function (err, data) {
            if (err) {
                future.throw(err);
            }
            else {
                _this.$logger.out("Installed ", packageName);
                future.return(data);
            }
        });
        return future;
    };
    NodePackageManager.prototype.isPackageCached = function (packagePath) {
        return this.$fs.exists(packagePath);
    };
    NodePackageManager.prototype.isPackageUnpacked = function (packagePath) {
        var _this = this;
        return (function () {
            return _this.$fs.getFsStats(packagePath).wait().isDirectory() && _this.$fs.enumerateFilesInDirectorySync(packagePath).length > 1;
        }).future()();
    };
    NodePackageManager.prototype.addToCacheCore = function (packageName, version) {
        var future = new Future();
        npm.commands["cache"].add(packageName, version, undefined, function (err, data) {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    };
    NodePackageManager.prototype.cacheUnpack = function (packageName, version, unpackTarget) {
        var future = new Future();
        unpackTarget = unpackTarget || path.join(npm.cache, packageName, version, "package");
        npm.commands["cache"].unpack(packageName, version, unpackTarget, function (err, data) {
            if (err) {
                future.throw(err);
            }
            else {
                future.return();
            }
        });
        return future;
    };
    NodePackageManager.prototype.getAvailableVersions = function (packageName) {
        var _this = this;
        return (function () {
            if (!_this.versionsCache[packageName]) {
                var url = NodePackageManager.NPM_REGISTRY_URL + packageName;
                var response = _this.$httpClient.httpRequest(url).wait().body;
                var json = JSON.parse(response);
                _this.versionsCache[packageName] = _.keys(json.versions);
            }
            return _this.versionsCache[packageName];
        }).future()();
    };
    NodePackageManager.NPM_LOAD_FAILED = "Failed to retrieve data from npm. Please try again a little bit later.";
    NodePackageManager.NPM_REGISTRY_URL = "http://registry.npmjs.org/";
    return NodePackageManager;
})();
exports.NodePackageManager = NodePackageManager;
$injector.register("npm", NodePackageManager);
